@@ -1,7 +1,7 @@
 /*
  *  This file is part of Poedit (https://poedit.net)
  *
- *  Copyright (C) 1999-2020 Vaclav Slavik
+ *  Copyright (C) 1999-2021 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -735,6 +735,14 @@ std::shared_ptr<SourceCodeSpec> Catalog::GetSourceCodeSpec() const
     for (auto& kv: m_header.GetAllHeaders())
         spec->XHeaders[kv.Key] = kv.Value;
 
+    // parse file type mapping (e.g. "h=gettext:c++")
+    wxStringTokenizer mapping(m_header.GetHeader("X-Poedit-Mapping"), ";");
+    while (mapping.HasMoreTokens())
+    {
+        auto m = mapping.GetNextToken();
+        spec->TypeMapping.emplace_back(m.BeforeFirst('='), m.AfterFirst('='));
+    }
+
     return spec;
 }
 
